#!/usr/bin/env python
# Python 3.*

import sys
import os
import shlex
from termcolor import colored

#Constants
LENGTH     = 0.1    #Motor displacement (Actually, extruding disantce with a ratio depending on the firmware)
FEEDRATE   = 10     #Motor speed
INTERVAL   = 2000    #Waiting time in ms
EXPRESSION = 'G1 Z'  #Expresion to identify the line after which the photo should be taken

class Writer:
    def __init__(self, path):
        self.path = path
        oldPath = self.createBackup()
        self.oldFile = open(oldPath, 'r')
        self.newFile = open(self.path, 'a')

    def createBackup(self):
        oldPath = self.path + '.bak'
        while os.path.exists(oldPath):
            oldPath += '.bak'
        os.rename(self.path, oldPath)
        return oldPath

class GcodeLine:
    def __init__(self, line, writer):
        self.line = line
        self.writer = writer
        
    def write(self):
        output = self.line
        if not "\n" in self.line:
            output += '\n'
        self.writer.newFile.write(output)

class App:
    def __init__(self, length, feedrate, interval, expression):
        if (len(sys.argv) <= 1):
            print(colored('Please provide a target file','yellow'))
            print()
            exit(0)
        else:
            self.length        = str(length)
            self.feedrate      = str(feedrate)
            self.interval      = str(interval)
            self.expression    = str(expression)
            self.oldValue      = {}
            self.oldValue['Y'] = 0
            self.oldValue['X'] = 0
            self.oldValue['F'] = self.feedrate
            
            self.writer = Writer(sys.argv[1])
            self.isColdExtrusionPrevented = True
            self.photoCount = 0
            self.main()
            
            # #Disable cold extrusion prevention
    
    def writeLine(self, line):    
        GcodeLine(line, self.writer).write()
    
    def writeStaticBlock(self):
        self.writeLine(';Stepper motor photo #' + str(self.photoCount))
        self.writeLine('T1')                      #switch to extruder 1
        self.writeLine('G1 Y0')                   #home Y axis
        self.writeLine('G1 X0')                   #home X axis
        self.writeLine('G1 F' + self.feedrate)       #set feedrate
        self.writeLine('G1 E' + self.length)      #move the stepper-motor
        self.writeLine('G4 P' + self.interval)    #wait
        self.writeLine('G1 E0')                   #move back the stepper-motor
        self.writeLine('G1 F' + str(self.oldValue['F'])) #set back feedrate
        self.writeLine('G1 X' + str(self.oldValue['X'])) #move back to previous X value
        self.writeLine('G1 Y' + str(self.oldValue['Y'])) #move back to previous Y value
        self.writeLine('T0')                      #switch to extruder 0
        self.writeLine(';Stepper motor photo - end')
         
    def writeBlock(self):
        if(self.isColdExtrusionPrevented):
            self.writeLine('M302') #disable cold extrusion prevention
            self.isColdExtrusionPrevented = False 
        self.writeStaticBlock()

    def writeLastBlock(self):
        self.writeStaticBlock()
        self.writeLine('M84')

    def main(self):
        for line in self.writer.oldFile:
            self.writeLine(line)
            self.readValue(line)
            if self.expression in line:
                self.writeBlock()
                self.photoCount += 1
        self.writeLastBlock()
        self.photoCount += 1

        print(self.photoCount)
        
    def readValue(self, line):
        for letter in ['X', 'Y', 'F']:
            if letter in line:
                items = shlex.split(line)
                for item in items:
                    if letter in item:
                        value = item[1:]
                        digitValue = value.replace('.', '') 
                        if digitValue.isdigit():
                            self.oldValue[letter] = value
                        
    
App(LENGTH, FEEDRATE, INTERVAL, EXPRESSION)
